import datetime
import time

from requests_oauthlib import OAuth1Session
from selenium.webdriver import Chrome, ChromeOptions
from selenium.webdriver.common.keys import Keys
from tqdm import tqdm

request_token_url = "https://api.zaim.net/v2/auth/request"
authorize_url = "https://auth.zaim.net/users/auth"
access_token_url = "https://api.zaim.net/v2/auth/access"
callback_uri = "https://www.zaim.net/"


def get_access_token():
    consumer_id = input("Please input consumer ID : ")
    consumer_secret = input("Please input consumer secret : ")
    print("\n")

    auth = OAuth1Session(
        client_key=consumer_id, client_secret=consumer_secret, callback_uri=callback_uri
    )

    auth.fetch_request_token(request_token_url)

    # Redirect user to zaim for authorization
    authorization_url = auth.authorization_url(authorize_url)
    print("Please go here and authorize : ", authorization_url)

    oauth_verifier = input("Please input oauth verifier : ")
    access_token_res = auth.fetch_access_token(
        url=access_token_url, verifier=oauth_verifier
    )
    access_token = access_token_res.get("oauth_token")
    access_token_secret = access_token_res.get("oauth_token_secret")

    print("\n")
    print("access token : {}".format(access_token))
    print("access token secret : {}".format(access_token_secret))
    print("oauth verifier : {}".format(oauth_verifier))


class ZaimAPI:
    def __init__(
        self,
        consumer_id,
        consumer_secret,
        access_token,
        access_token_secret,
        oauth_verifier,
    ):
        self.consumer_id = consumer_id
        self.consumer_secret = consumer_secret

        self.verify_url = "https://api.zaim.net/v2/home/user/verify"
        self.money_url = "https://api.zaim.net/v2/home/money"
        self.payment_url = "https://api.zaim.net/v2/home/money/payment"
        self.income_url = "https://api.zaim.net/v2/home/money/income"
        self.transfer_url = "https://api.zaim.net/v2/home/money/transfer"
        self.category_url = "https://api.zaim.net/v2/home/category"
        self.genre_url = "https://api.zaim.net/v2/home/genre"
        self.account_url = "https://api.zaim.net/v2/home/account"
        self.currency_url = "https://api.zaim.net/v2/currency"

        self.auth = OAuth1Session(
            client_key=self.consumer_id,
            client_secret=self.consumer_secret,
            resource_owner_key=access_token,
            resource_owner_secret=access_token_secret,
            callback_uri=callback_uri,
            verifier=oauth_verifier,
        )

        self._build_id_table()

    def verify(self):
        return self.auth.get(self.verify_url).json()

    def get_data(self):
        return self.auth.get(self.money_url).json()["money"]

    def insert_payment_simple(
        self,
        date,
        amount,
        genre,
        from_account=None,
        comment=None,
        name=None,
        place=None,
    ):
        genre_id = self.genre_stoi[genre]
        category_id = self.genre_to_category[genre_id]
        if from_account is not None:
            from_account_id = self.account_stoi[from_account]
        else:
            from_account_id = None
        return self.insert_payment(
            date, amount, category_id, genre_id, from_account_id, comment, name, place
        )

    def insert_payment(
        self,
        date,
        amount,
        category_id,
        genre_id,
        from_account_id=None,
        comment=None,
        name=None,
        place=None,
    ):
        data = {
            "mapping": 1,
            "category_id": category_id,
            "genre_id": genre_id,
            "amount": amount,
            "date": date.strftime("%Y-%m-%d"),
        }
        if from_account_id is not None:
            data["from_account_id"] = from_account_id
        if comment is not None:
            data["comment"] = comment
        if name is not None:
            data["name"] = name
        if place is not None:
            data["place"] = place
        return self.auth.post(self.payment_url, data=data)

    def update_payment_simple(
        self,
        data_id,
        date,
        genre,
        amount,
        from_account=None,
        comment=None,
        name=None,
        place=None,
    ):
        genre_id = self.genre_stoi[genre]
        category_id = self.genre_to_category[genre_id]
        if from_account is not None:
            from_account_id = self.account_stoi[from_account]
        else:
            from_account_id = None
        return self.update_payment(
            data_id,
            date,
            amount,
            category_id,
            genre_id,
            from_account_id,
            comment,
            name,
            place,
        )

    def update_payment(
        self,
        data_id,
        date,
        amount,
        category_id,
        genre_id,
        from_account_id=None,
        comment=None,
        name=None,
        place=None,
    ):
        data = {
            "mapping": 1,
            "id": data_id,
            "category_id": category_id,
            "genre_id": genre_id,
            "amount": amount,
            "date": date.strftime("%Y-%m-%d"),
        }
        if from_account_id is not None:
            data["from_account_id"] = from_account_id
        if comment is not None:
            data["comment"] = comment
        if name is not None:
            data["name"] = name
        if place is not None:
            data["place"] = place
        return self.auth.put("{}/{}".format(self.payment_url, data_id), data=data)

    def delete_payment(self, data_id):
        return self.auth.delete("{}/{}".format(self.payment_url, data_id))

    def insert_income_simple(
        self, date, category, amount, to_account=None, comment=None, place=None
    ):
        category_id = self.category_stoi[category]
        if to_account is not None:
            to_account_id = self.account_stoi[to_account]
        else:
            to_account_id = None
        return self.insert_income(
            date, category_id, amount, to_account_id, comment, place
        )

    def insert_income(
        self, date, category_id, amount, to_account_id=None, comment=None, place=None
    ):
        data = {
            "mapping": 1,
            "category_id": category_id,
            "amount": amount,
            "date": date.strftime("%Y-%m-%d"),
        }
        if to_account_id is not None:
            data["to_account_id"] = to_account_id
        if comment is not None:
            data["comment"] = comment
        if place is not None:
            data["place"] = place
        return self.auth.post(self.income_url, data=data)

    def update_income_simple(
        self, data_id, date, category, amount, to_account=None, comment=None, place=None
    ):
        category_id = self.category_stoi[category]
        if to_account is not None:
            to_account_id = self.account_stoi[to_account]
        else:
            to_account_id = None
        return self.update_income(
            data_id, date, category_id, amount, to_account_id, comment, place
        )

    def update_income(
        self,
        data_id,
        date,
        category_id,
        amount,
        to_account_id=None,
        comment=None,
        place=None,
    ):
        data = {
            "mapping": 1,
            "id": data_id,
            "category_id": category_id,
            "amount": amount,
            "date": date.strftime("%Y-%m-%d"),
        }
        if to_account_id is not None:
            data["to_account_id"] = to_account_id
        if comment is not None:
            data["comment"] = comment
        if place is not None:
            data["place"] = place
        return self.auth.put("{}/{}".format(self.income_url, data_id), data=data)

    def delete_income(self, data_id):
        return self.auth.delete("{}/{}".format(self.income_url, data_id))

    def insert_transfer_simple(
        self, date, amount, from_account, to_account, comment=None
    ):
        from_account_id = self.account_stoi[from_account]
        to_account_id = self.account_stoi[to_account]
        return self.insert_transfer(
            date, amount, from_account_id, to_account_id, comment
        )

    def insert_transfer(
        self, date, amount, from_account_id, to_account_id, comment=None
    ):
        data = {
            "mapping": 1,
            "amount": amount,
            "date": date.strftime("%Y-%m-%d"),
            "from_account_id": from_account_id,
            "to_account_id": to_account_id,
        }
        if comment is not None:
            data["comment"] = comment
        return self.auth.post(self.transfer_url, data=data)

    def update_transfer_simple(
        self, data_id, date, amount, from_account, to_account, comment=None
    ):
        from_account_id = self.account_stoi[from_account]
        to_account_id = self.account_stoi[to_account]
        return self.update_transfer(
            data_id, date, amount, from_account_id, to_account_id, comment
        )

    def update_transfer(
        self, data_id, date, amount, from_account_id, to_account_id, comment=None
    ):
        data = {
            "mapping": 1,
            "id": data_id,
            "amount": amount,
            "date": date.strftime("%Y-%m-%d"),
            "from_account_id": from_account_id,
            "to_account_id": to_account_id,
        }
        if comment is not None:
            data["comment"] = comment
        return self.auth.put("{}/{}".format(self.transfer_url, data_id), data=data)

    def delete_transfer(self, data_id):
        return self.auth.delete("{}/{}".format(self.transfer_url, data_id))

    def _build_id_table(self):
        self.genre_itos = {}
        self.genre_stoi = {}
        self.genre_to_category = {}
        genre = self._get_genre()["genres"]
        for g in genre:
            self.genre_itos[g["id"]] = g["name"]
            self.genre_stoi[g["name"]] = g["id"]
            self.genre_to_category[g["id"]] = g["category_id"]
        self.category_itos = {}
        self.category_stoi = {}
        category = self._get_category()["categories"]
        for c in category:
            self.category_itos[c["id"]] = c["name"]
            self.category_stoi[c["name"]] = c["id"]
        self.account_stoi = {}
        self.account_itos = {}
        account = self._get_account()["accounts"]
        for a in account:
            self.account_itos[a["id"]] = a["name"]
            self.account_stoi[a["name"]] = a["id"]

    def _get_account(self):
        return self.auth.get(self.account_url).json()

    def _get_category(self):
        return self.auth.get(self.category_url).json()

    def _get_genre(self):
        return self.auth.get(self.genre_url).json()


class ZaimCrawler:
    def __init__(self, user_id, password, driver_path=None, headless=False):
        options = ChromeOptions()
        options.add_argument("--disable-gpu")
        options.add_argument("--no-sandbox")
        options.add_argument('--disable-dev-shm-usage')
        options.add_argument('--remote-debugging-port=9222')
        if headless:
            options.add_argument("--headless")
        if driver_path is not None:
            self.driver = Chrome(executable_path=driver_path, options=options)
        else:
            self.driver = Chrome(options=options)
        self.driver.set_window_size(640, 480)
        print("Start Chrome Driver.")
        print("Login to Zaim.")

        self.driver.get("https://auth.zaim.net/")
        time.sleep(1)

        self.driver.find_element_by_id("UserEmail").send_keys(user_id)
        self.driver.find_element_by_id("UserPassword").send_keys(password, Keys.ENTER)
        time.sleep(1)
        print("Login Success.")

    def get_data(self, year, month, progress=True):
        month = str(month).zfill(2)
        print("Get Data of {}/{}.".format(year, month))
        self.driver.get("https://zaim.net/money?month={}{}".format(year, month))
        time.sleep(1)

        table = self.driver.find_element_by_class_name("money-list")
        lines = table.find_elements_by_tag_name("tr")

        print("Found {} data.".format(len(lines)))
        if progress:
            pbar = tqdm(total=len(lines))

        data = []
        for line in reversed(lines):
            items = line.find_elements_by_tag_name("td")

            item = {}
            item["id"] = (
                items[0]
                .find_element_by_tag_name("a")
                .get_attribute("data-url")
                .split("/")[2]
            )
            item["type"] = (
                items[1]
                .find_element_by_tag_name("a")
                .get_attribute("class")
                .split(" ")[1]
            )
            item["count"] = (
                items[1]
                .find_element_by_tag_name("i")
                .get_attribute("title")
                .split("（")[0]
            )
            date = items[2].text.split("（")[0]
            item["date"] = datetime.datetime.strptime(
                "{}年{}".format(year, date), "%Y年%m月%d日"
            )
            item["category"] = (
                items[3].find_element_by_tag_name("span").get_attribute("data-title")
            )
            item["genre"] = items[3].find_element_by_tag_name("a").text
            item["amount"] = int(items[4].text.strip("¥").replace(",", ""))
            m_from = items[5].find_elements_by_tag_name("img")
            if len(m_from) != 0:
                item["from_account"] = m_from[0].get_attribute("data-title")
            m_to = items[6].find_elements_by_tag_name("img")
            if len(m_to) != 0:
                item["to_account"] = m_to[0].get_attribute("data-title")
            item["place"] = (
                items[7].find_element_by_tag_name("span").get_attribute("title")
            )
            item["name"] = (
                items[8].find_element_by_tag_name("span").get_attribute("title")
            )
            item["comment"] = (
                items[9].find_element_by_tag_name("span").get_attribute("title")
            )
            data.append(item)
            if progress:
                pbar.update(1)
        if progress:
            pbar.close()
        return data

    def close(self):
        self.driver.close()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyzaim']

package_data = \
{'': ['*'], 'pyzaim': ['.vscode/*']}

install_requires = \
['requests-oauthlib>=1.3.0,<2.0.0',
 'selenium>=3.141.0,<4.0.0',
 'tqdm>=4.43.0,<5.0.0']

setup_kwargs = {
    'name': 'pyzaim',
    'version': '1.0.5',
    'description': 'Zaimのデータを取得・操作するPythonパッケージ',
    'long_description': "# pyzaim\n\n[Zaim](https://zaim.net/)のデータを取得・操作するPythonパッケージ\n\n## 概要\n\n大きくわけて2つの処理を行うパッケージです。\n\n- [Zaim API](https://dev.zaim.net/)のラッパークラス\n  - Zaim APIのアクセストークンの発行\n  - Rest APIとして提供されている処理の実行\n- [Selenium](https://github.com/SeleniumHQ/selenium/tree/master/py)を用いたデータ取得\n  - Zaimにはクレジットカードや銀行口座から自動でデータ取得する機能があるが、APIではそれらのデータにはアクセスできない\n  - これらの情報を取得するため、Seleniumのwebdriver(Chrome)を用いてデータを取得\n\n## インストール\n\n```bash\npip install pyzaim\n```\n\n## 準備\n\n- Zaimアカウントの作成\n- Zaim Developersでのアプリケーションの登録 (コンシューマID、コンシューマシークレットの発行)\n- Google Chromeおよびseleniumの導入\n\n## 使い方\n\n### Zaim APIのラッパークラスの使い方\n\n- アクセストークンの発行\n\n```python\nfrom pyzaim import get_access_token()\n\nget_access_token()\n\n# コンシューマIDとコンシューマシークレットを聞かれるので入力\n# 認証ページのURLが表示されるので、アクセスして許可\n# 遷移先ページのソースコードから「oauth_verifier」と書いてあるコードをコピーして入力\n# 問題なければアクセストークンとアクセスシークレットが表示される\n```\n\n- APIを利用してデータを取得・操作\n\n```python\nfrom pyzaim import ZaimAPI\n\napi = ZaimAPI('コンシューマID', 'コンシューマシークレット',\n              'アクセストークン', 'アクセスシークレット', 'verifier')\n\n# 動作確認 (ユーザーID等のデータが取得されて、表示されればOK)\nprint(api.verify())\n\n# データの取得\ndata = api.get_data()\n\n# 支払いデータの登録\napi.insert_payment_simple('日付(datetime.date型)', '金額(int)', 'ジャンル名',\n                          '口座名', 'コメント', '品名', '店舗名') # 後半4つは任意入力\n\n# 使用できるジャンル名は以下で確認できる\nprint(api.genre_itos)\n\n# 使用できる口座名は以下で確認できる\nprint(api.account_itos)\n\n# 支払いデータの更新 (更新対象データのIDはapi.get_data()で確認)\napi.update_payment_simple('更新対象データのID', '日付(datetime.date型)', '金額(int)',\n                          'ジャンル名', '口座名', 'コメント', '品名', '店舗名') # 後半4つは任意入力\n\n# 支払いデータの削除\napi.delete_payment('削除対象のデータのID')\n```\n\n### seleniumを用いたデータ取得\n\n```python\nfrom pyzaim import ZaimCrawler\n\n# Chrome Driverの起動とZaimへのログイン、ログインには少し時間がかかります\nclawler = ZaimCrawler('ログインID', 'ログインパスワード', \n                    driver_path='Chrome Driverのパス'(PATHが通っていれば省略可),\n                    headless=False) # headlessをTrueにするとブラウザを表示できる\n\n# データの取得 (データの取得には少し時間がかかります、時間はデータ件数による)\ndata = crawler.get_data('取得する年(int)', '取得する月(int)', progress=True) # progressをFalseにするとプログレスバーを非表示にできる\n\n# 終了処理\ncrawler.close()\n```\n\n",
    'author': 'reeve0930',
    'author_email': 'reeve0930@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/reeve0930/pyzaim',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
